// VISensorDataProcess.h: interface of the CVISensorDataProcess class
//
/*--------------------------------------------------------------------------*/
/*       Copyright(C) 2004-2008 KONICA MINOLTA SENSING,INC.                 */
/*--------------------------------------------------------------------------*/

#if !defined(__VI_SENSOR_DATAPROCESS_H__)
#define __VI_SENSOR_DATAPROCESS_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifdef DATAPROCESS_EXPORTS
#define DATAPROCESS_API __declspec(dllexport)
#else
#define DATAPROCESS_API __declspec(dllimport)
#endif

#include "VIVertexData.h"
#include <VISensorRawData.h>
#include "VISensorPolygonProcess.h"
#include "VISensorVertexProcess.h"
#include "VISensorMonoProcess.h"
#include "VISensorColorProcess.h"
#include "VISensorColorData.h"
#include "VISensorMonoData.h"

class CVISensorRawDataInfo;
class DATAPROCESS_API CVISensorDataProcess  
{
public:
    virtual BOOL Convert3d(VICOORD2D &ax2, VICOORD3D &axs) = 0;
    virtual BOOL Attach(CVISensorRawData *pRawData,CVISensorImportPara *pImportPara = NULL) = 0;
	virtual BOOL GetPolygonData(CVISensorRawData* pRawData,CVIObject& Polygon,CVISensorImportPara* pImportPara = NULL,LPCTSTR name = NULL); // const char* -> LPCTSTR
	virtual BOOL GetVertexData(CVISensorRawData* pRawData,CVIVertexData *pVertexData,CVISensorImportPara* pImportPara = NULL);
    CVISensorDataProcess();
	virtual ~CVISensorDataProcess();
    virtual BOOL CopyColorData(const CVISensorRawData* pSrcRawData, CVISensorRawData* pDestRawData) = 0;//KH@060629
    virtual BOOL GetVvdImageData(CVISensorRawData* pRawData,CVvdImage& VvdImage,int DataLabel,CVISensorImportPara* pImportPara = NULL) = 0;
	virtual BOOL SensorColorToVvdImage(CVISensorColorData* pColorData,CVvdImage& VvdImage,CVISensorImportPara* pImportPara);
	virtual BOOL SensorMonoToVvdImage(CVISensorMonoData* pMonoData,CVvdImage& VvdImage,CVISensorImportPara* pImportPara);
	virtual BOOL SetImageToPolygon(CVISensorRawData* pRawData,CVIObject& Polygon,CVISensorImportPara* pImportPara);
    virtual BOOL CoordTransPolygon(CVISensorRawData* pRawData,CVIObject& Polygon) = 0;
	virtual BOOL RawDataToVertexData(CVISensorRawData* pRawData,CVIVertexData *pVertexData,CVISensorImportPara* pImportPara);
	virtual BOOL VertexToPolygon(CVISensorRawData* pRawData,CVIVertexData* pVertexData,CVIObject& Polygon,CVISensorImportPara* pImportPara,LPCTSTR name); // const char* -> LPCTSTR
	virtual BOOL GetDefaultImportPara(CVISensorRawData* pRawData,CVISensorImportPara* pImportPara) = 0;
    virtual BOOL GetRawDataInfo(CVISensorRawData* pRawData,CVISensorRawDataInfo* pRawDataInfo) = 0;
protected:
	virtual BOOL CoordTransVertexData(CVISensorRawData* pRawData,CVIVertexData *pVertexData) = 0;
	CVISensorMonoProcess* m_pMonoProcess;
	CVISensorPolygonProcess* m_pPolygonProcess;
	CVISensorVertexProcess* m_pVertexProcess;
	CVISensorColorProcess* m_pColorProcess;
    int     m_nErrCode;
private:
public:
	virtual BOOL GetErrCode(int& nErrCode);
	virtual void SetErrCode(int nErrCode);
};

#endif
