/*--------------------------------------------------------------------------*/
/*       Copyright(C) 2004-2008 KONICA MINOLTA SENSING,INC.                 */
/*--------------------------------------------------------------------------*/
#include "StdAfx.h"
#include ".\VI9iDataProcessControl.h"
#include <VICreator.h>
#include <VividImportPara.h>
#include <Vivid9iErrCode.h>
#include <VIParameter.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

//---------------------------------------------------------------------------------------//
CVI9iDataProcessControl::CVI9iDataProcessControl(void)
{
}

//---------------------------------------------------------------------------------------//
CVI9iDataProcessControl::~CVI9iDataProcessControl(void)
{
}
//---------------------------------------------------------------------------------------//
void CVI9iDataProcessControl::SetErrCode(int nErrCode)
{
    m_nErrCode = nErrCode;
}

//---------------------------------------------------------------------------------------//
int CVI9iDataProcessControl::GetErrCode(void)
{
    return m_nErrCode;
}

//---------------------------------------------------------------------------------------//
// Save Polygon file
BOOL CVI9iDataProcessControl::SavePolygonFile(void)
{
	// Setting the import parameter
    CVividImportPara VIVIDImportPara;
    VIVIDImportPara.m_nReduce   = VIReduceAdaptive          ;
    VIVIDImportPara.m_nFilter   = VINoiseFilterHighQuality	;
    VIVIDImportPara.m_nCutData  = VINoneCut		            ;
    VIVIDImportPara.m_nFillHole = VIFillOff		            ;

    // Getting the import file name
    CFileDialog fileDlg1(TRUE,_T("cdk"),NULL,OFN_FILEMUSTEXIST,_T("VIVID9i File (*.cdk)|*.cdk|VIVID910 File (*.cdm)|*.cdm||"), NULL);
    if( fileDlg1.DoModal() != IDOK ) return TRUE;
    CWaitCursor cursor;
    CString filename = fileDlg1.GetPathName();

	TCHAR drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME],ext[_MAX_EXT];
	_tsplitpath_s( filename, drive, dir, fname, ext );


	CVISensorRawFileProcess* pRawFileProcess;

	if( 0 == _tcsicmp( ext, _T(".CDK") ) ) {
		pRawFileProcess = VISensorRawFileProcessCreate(VI_VIVID9i_SENSOR);
	}
	else if( 0== _tcsicmp( ext, _T(".CDM") ) ) {
		pRawFileProcess = VISensorRawFileProcessCreate(VI_VIVID910_SENSOR);
		VIVIDImportPara.m_nReduce   = VIReduceNone;
		VIVIDImportPara.m_nFilter   = VINoiseFilter;
	}
	else {
		return FALSE;
	}


    // Import the CDK file
    BOOL bSuccess; 
    bSuccess = pRawFileProcess->ReadSensorRawData(filename);
    if (!bSuccess)
    {
        int nErrCode;
        pRawFileProcess->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
		VISensorRawFileProcessDelete(pRawFileProcess);
		pRawFileProcess = NULL;
        return FALSE;
    }
	

    //Get PolygonList
    CVIObjectList PolygonList;  
    bSuccess = pRawFileProcess->GetPolygonList(&PolygonList,&VIVIDImportPara, _T("TestData") );
    if (!bSuccess)
    {
        int nErrCode;
        pRawFileProcess->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
		VISensorRawFileProcessDelete(pRawFileProcess);
		pRawFileProcess = NULL;
        return FALSE;
    }

	// Save to VVD file
    CVISensorPolygonFileProcess* pPolygonFileProcess;
    CString PolygonFileName;
    
	if( 0 == _tcsicmp( ext, _T(".CDK") ) ) {
		pPolygonFileProcess = VISensorPolygonFileProcessCreate(VI_VIVID9i_SENSOR);
	}
	else if( 0== _tcsicmp( ext, _T(".CDM") ) ) {
		pPolygonFileProcess = VISensorPolygonFileProcessCreate(VI_VIVID910_SENSOR);
	}
	else {
		VISensorRawFileProcessDelete(pRawFileProcess);
		pRawFileProcess = NULL;
		return FALSE;
	}

    filename.Replace(ext, _T(""));
    int nDataNum;
    pRawFileProcess->GetDataNum(nDataNum);
    for (int i = 0;i < nDataNum; i++)
    {
        PolygonFileName.Format( _T("%s-%d.vvd"),filename,i);
        bSuccess = pPolygonFileProcess->SavePolygonData(*PolygonList.m_objects[i],PolygonFileName);
        if (!bSuccess)
        {
            int nErrCode;
            pPolygonFileProcess->GetErrCode(nErrCode);
            SetErrCode(nErrCode);
			VISensorPolygonFileProcessDelete(pPolygonFileProcess);
			pPolygonFileProcess = NULL;
			VISensorRawFileProcessDelete(pRawFileProcess);
			pRawFileProcess = NULL;
            return FALSE;
        }
    }
    
	VISensorPolygonFileProcessDelete(pPolygonFileProcess);
	pPolygonFileProcess = NULL;
	VISensorRawFileProcessDelete(pRawFileProcess);
	pRawFileProcess = NULL;

    return TRUE;
}

//---------------------------------------------------------------------------------------//
//Save ASCII file
BOOL CVI9iDataProcessControl::SaveASCIIData(void)
{
	// Setting the import parameter
    CVividImportPara VIVIDImportPara;
    VIVIDImportPara.m_nReduce   = VIReduceAdaptive          ;
    VIVIDImportPara.m_nFilter   = VINoiseFilterHighQuality	;
    VIVIDImportPara.m_nCutData  = VINoneCut		            ;
    VIVIDImportPara.m_nFillHole = VIFillOff		            ;

	// Getting the import file name
    CFileDialog fileDlg1(TRUE,_T("cdk"),NULL,OFN_FILEMUSTEXIST,_T("VIVID9i File (*.cdk)|*.cdk|VIVID910 File (*.cdm)|*.cdm||"), NULL);
    if( fileDlg1.DoModal() != IDOK ) return TRUE;
    CWaitCursor cursor;
    CString filename = fileDlg1.GetPathName();

	TCHAR drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME],ext[_MAX_EXT];
	_tsplitpath_s( filename, drive, dir, fname, ext );

	CVISensorRawFileProcess* pRawFileProcess;

	if( 0 == _tcsicmp( ext, _T(".CDK") ) ) {
		pRawFileProcess = VISensorRawFileProcessCreate(VI_VIVID9i_SENSOR);
	}
	else if( 0== _tcsicmp( ext, _T(".CDM") ) ) {
		pRawFileProcess = VISensorRawFileProcessCreate(VI_VIVID910_SENSOR);
		VIVIDImportPara.m_nReduce   = VIReduceNone;
		VIVIDImportPara.m_nFilter   = VINoiseFilter;
	}
	else {
		return FALSE;
	}

    // Import the CDK file
    BOOL bSuccess; 
    bSuccess = pRawFileProcess->ReadSensorRawData(filename);
    if (!bSuccess)
    {
        int nErrCode;
        pRawFileProcess->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
		VISensorRawFileProcessDelete(pRawFileProcess);
		pRawFileProcess = NULL;
        return FALSE;
    }
	

    //Getting the VertexDataList
    CVIVertexDataList VertexList;
    bSuccess = pRawFileProcess->GetVertexDataList(&VertexList,&VIVIDImportPara);
    if (!bSuccess)
    {
        int nErrCode;
        pRawFileProcess->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
		VISensorRawFileProcessDelete(pRawFileProcess);
		pRawFileProcess = NULL;
        return FALSE;
    }

    //Save ASCII File
    filename.Replace(ext, _T(""));
    int nDataNum;
    CString ASCIIFileName;
    pRawFileProcess->GetDataNum(nDataNum);
    for (int i = 0;i < nDataNum; i++)
    {
        ASCIIFileName.Format(_T("%s-%d.asc"),filename,i);
        bSuccess = SaveVertexData(*VertexList.m_VertexData[i],(LPCTSTR)ASCIIFileName);
		if (!bSuccess) {
			VISensorRawFileProcessDelete(pRawFileProcess);
			pRawFileProcess = NULL;
            return FALSE;
		}
    }

	VISensorRawFileProcessDelete(pRawFileProcess);
	pRawFileProcess = NULL;
    return TRUE;
}

//---------------------------------------------------------------------------------------//
//Save VertexData (ASCII)
BOOL CVI9iDataProcessControl::SaveVertexData(CVIVertexData& VertexData,const char* filename)
{
	CFile fl;
	if (!fl.Open( filename,	CFile::modeCreate | CFile::modeWrite))
	{
		SetErrCode(VI_ERR_FILE_WRITE);
        return FALSE;
    }
    CArchive ar(&fl, CArchive::store);

    //Setting Header
    CString strHeader;
    int width,height;
    VertexData.GetSize(width,height);
    strHeader.Format( _T("# vivid 3D Data : \nwidth %d\nheight %d\n"),width,height);
    ar.WriteString( strHeader );
    // Write Vertex Data
    float x, y, z;
    CString strLine;
    for ( int i = 0;i < height; i++)
    {
        for ( int j = 0;j < width; j++)
        {
            x = (float)VertexData.m_pVertex[i * width + j].point.x;
            y = (float)VertexData.m_pVertex[i * width + j].point.y;
            z = (float)VertexData.m_pVertex[i * width + j].point.z;
            if ((x == VI_INVALID_DATA_3D) || (y == VI_INVALID_DATA_3D)||(y == VI_INVALID_DATA_3D))  continue;
            strLine.Format( _T("%.3f %.3f %.3f\r\n"), x, y, z);
            ar.WriteString( strLine );
        }
    }
    //Setting footer
    CString strFooter;
    strFooter = _T("**end cloud**\n");
    ar.WriteString( strFooter );

    ar.Close();
	fl.Close();
    return TRUE;
}

//---------------------------------------------------------------------------------------//
//Read Polygon file
BOOL CVI9iDataProcessControl::ReadPolygonFile(CVIObject& PolygonData)
{
    // Getting the Read vvd file name
    CFileDialog fileDlg1(TRUE,_T("vvd"),NULL,OFN_FILEMUSTEXIST,_T("Polygon Data Files (*.vvd)|*.vvd||"), NULL);
    if( fileDlg1.DoModal() != IDOK ) return TRUE;
    CWaitCursor cursor;
    CString filename = fileDlg1.GetPathName();


    CVISensorPolygonFileProcess* pPolygonFileProcess;
    pPolygonFileProcess = VISensorPolygonFileProcessCreate(VI_VIVID9i_SENSOR);

    //Read vvd file
    BOOL bSuccess; 
    bSuccess = pPolygonFileProcess->ReadPolygonData(PolygonData,(LPCTSTR)filename);
    if (!bSuccess)
    {
        int nErrCode;
        pPolygonFileProcess->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
		VISensorPolygonFileProcessDelete(pPolygonFileProcess);
		pPolygonFileProcess = NULL;
        return FALSE;
    }

    AfxMessageBox( _T("Succeeded."), MB_OK | MB_ICONINFORMATION );

	VISensorPolygonFileProcessDelete(pPolygonFileProcess);
	pPolygonFileProcess = NULL;
    return TRUE;
}
//---------------------------------------------------------------------------------------//
//Get CDK Color Data (first Data)
BOOL CVI9iDataProcessControl::GetColorData(CVvdImage& ImageData, const BOOL bLog, const BOOL bDark)
{
    // Getting the import file name
    CFileDialog fileDlg1(TRUE,_T("cdk"),NULL,OFN_FILEMUSTEXIST,_T("VIVID9i File (*.cdk)|*.cdk|VIVID910 File (*.cdm)|*.cdm||"), NULL);
    if( fileDlg1.DoModal() != IDOK ) return TRUE;
    CWaitCursor cursor;
    CString filename = fileDlg1.GetPathName();

	TCHAR drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME],ext[_MAX_EXT];
	_tsplitpath_s( filename, drive, dir, fname, ext );

	CVISensorRawFileProcess* pRawFileProcess;

	if( 0 == _tcsicmp( ext, _T(".CDK") ) ) {
		pRawFileProcess = VISensorRawFileProcessCreate(VI_VIVID9i_SENSOR);
	}
	else if( 0== _tcsicmp( ext, _T(".CDM") ) ) {
		pRawFileProcess = VISensorRawFileProcessCreate(VI_VIVID910_SENSOR);
	}
	else {
		return FALSE;
	}

    // Import the CDK file
    BOOL bSuccess; 
    bSuccess = pRawFileProcess->ReadSensorRawData(filename);
    if (!bSuccess)
    {
        int nErrCode;
        pRawFileProcess->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
		VISensorRawFileProcessDelete(pRawFileProcess);
		pRawFileProcess = NULL;
        return FALSE;
    }

    //Getting the Color Data
    int nDataNum = 0;
    CVividImportPara VIVIDImportPara;
    VIVIDImportPara.m_bDark = bDark ;
    VIVIDImportPara.m_bLog  = bLog  ;
	VIVIDImportPara.m_nReduce = VIReduceNone; // For 910
    bSuccess = pRawFileProcess->GetVvdImageData(ImageData,VIVID_RAW_COLOR_DATA,nDataNum,&VIVIDImportPara);
    if (!bSuccess)
    {
        int nErrCode;
        pRawFileProcess->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
		VISensorRawFileProcessDelete(pRawFileProcess);
		pRawFileProcess = NULL;
        return FALSE;
    }
	VISensorRawFileProcessDelete(pRawFileProcess);
	pRawFileProcess = NULL;
    return TRUE;
}
//---------------------------------------------------------------------------------------//
//Get CDK Reflect Data (first Data)
BOOL CVI9iDataProcessControl::GetReflectData(CVvdImage& ImageData)
{
    // Getting the import file name
    CFileDialog fileDlg1(TRUE,_T("cdk"),NULL,OFN_FILEMUSTEXIST,_T("VIVID9i File (*.cdk)|*.cdk|VIVID910 File (*.cdm)|*.cdm||"), NULL);
    if( fileDlg1.DoModal() != IDOK ) return TRUE;
    CWaitCursor cursor;
    CString filename = fileDlg1.GetPathName();

	TCHAR drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME],ext[_MAX_EXT];
	_tsplitpath_s( filename, drive, dir, fname, ext );

	CVISensorRawFileProcess* pRawFileProcess;

	if( 0 == _tcsicmp( ext, _T(".CDK") ) ) {
		pRawFileProcess = VISensorRawFileProcessCreate(VI_VIVID9i_SENSOR);
	}
	else if( 0== _tcsicmp( ext, _T(".CDM") ) ) {
		pRawFileProcess = VISensorRawFileProcessCreate(VI_VIVID910_SENSOR);
	}
	else {
		return FALSE;
	}

    // Import the CDK file
    BOOL bSuccess; 
    bSuccess = pRawFileProcess->ReadSensorRawData(filename);
    if (!bSuccess)
    {
        int nErrCode;
        pRawFileProcess->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
		VISensorRawFileProcessDelete(pRawFileProcess);
		pRawFileProcess = NULL;
        return FALSE;
    }

    //Getting the Reflect Data
    int nDataNum = 0;
    CVividImportPara VIVIDImportPara;   //LOG,DARK setting not supported
    bSuccess = pRawFileProcess->GetVvdImageData(ImageData,VIVID_RAW_REFLECT_DATA,nDataNum,&VIVIDImportPara);
    if (!bSuccess)
    {
        int nErrCode;
        pRawFileProcess->GetErrCode(nErrCode);
        SetErrCode(nErrCode);
		VISensorRawFileProcessDelete(pRawFileProcess);
		pRawFileProcess = NULL;
        return FALSE;
    }
	VISensorRawFileProcessDelete(pRawFileProcess);
	pRawFileProcess = NULL;
    return TRUE;
}
