// VvdImage.h: interface of the CVvdImage class
//
/*--------------------------------------------------------------------------*/
/*       Copyright(C) 2004-2008 KONICA MINOLTA SENSING,INC.                 */
/*--------------------------------------------------------------------------*/

#if !defined(__VVDIMAGE_H__)
#define __VVDIMAGE_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
#include <vividIItypes.h>
#include    <time.h>

typedef VVD_Ulong    VVD_Pixel;
#ifdef DATAPROCESS_EXPORTS
#define DATAPROCESS_API __declspec(dllexport)
#else
#define DATAPROCESS_API __declspec(dllimport)
#endif



typedef struct {
    VVD_Uchar	blue;
    VVD_Uchar	green;
    VVD_Uchar	red;
    VVD_Uchar	reserve;
} VVD_BGRA;
//-------------------------------------------------------

typedef struct {
    VVD_Uchar	red;
    VVD_Uchar	green;
    VVD_Uchar	blue;
    VVD_Uchar	alpha;
} VVD_RGBA;
//-------------------------------------------------------
typedef struct {
    VVD_Uchar	red;
    VVD_Uchar	green;
    VVD_Uchar	blue;
} VVD_RGB;
//-------------------------------------------------------
typedef struct {
    VVD_Float	red;
    VVD_Float	green;
    VVD_Float	blue;
} VVD_RGBF;
//-------------------------------------------------------

typedef struct {
    VVD_Uchar	padding;
    VVD_Uchar	blue;
    VVD_Uchar	green;
    VVD_Uchar	red;
} VVD_Color;
//-------------------------------------------------------

typedef enum {
    VVD_GREY_IMAGE,
    VVD_RGB_IMAGE,
    VVD_SYMBOL_IMAGE
}VVD_ImageType;
//-------------------------------------------------------

typedef struct{
    float	viewPoint[3];	    /* viewpoint */
    float	interest[3];	    /* interest */
    float	upVector[3];	    /* the upward direction of the color */
    float	focalLength;		/* focal length */
    float	pitchX, pitchY;	    /* pitch of pinxels on color CCD */
    float	centerX, centerY;	/* the optical axis on the color CCD */
	long	time;				/* the time when the the image was taken */
}VICameraInfo;
//-------------------------------------------------------

typedef struct{
	long	width, height;		/* image Size */
    double	viewPoint[3];		/* viewpoint */
    double	direction[3];		/* gaze vector */
    double	upVector[3];		/* the upward direction of the color */
    double	focalLength;		/* focal length */
	double	pitchX, pitchY;		/* pitch of pinxels on color CCD */
    double	centerX, centerY;	/* the optical axis on the color CCD */
}VIImageInfo;
//-------------------------------------------------------

//Ref VVDII_Image
class DATAPROCESS_API CVIImage
{
public:
    CVIImage& operator=( CVIImage & sc );
	BOOL Init(int width,int height);
    CVIImage();
    virtual ~CVIImage();
    VIImageInfo	    m_viewInfo;		/* camera parameters */
    long			m_width;		/* width of the color image */	//VVD_Int -> long
    long			m_height;		/* height of the color image */ //VVD_Int -> long
    VVD_Pixel		*m_pixels;		/* rasterized pixel array */
    VVD_ImageType	m_attribute;	/* type of color data */
};

//-------------------------------------------------------
class DATAPROCESS_API CVvdImage  
{
public:
	void GetSize(int& width,int& height);
	BOOL Init(int width,int height);
    VVD_BGRA* m_pImage;
	CVvdImage();
    CVvdImage(const CVvdImage& VvdImage);
    CVvdImage& operator = (const CVvdImage& VvdImage);
    friend class CVividImageData;
	virtual ~CVvdImage();
private:
    int m_nHeight;
	int m_nWidth;

protected:
    void Copy(const CVvdImage& VvdImage);
};

#endif
